/* Copyright 1998-2011 Research Foundation State University of New York */

/* This file is part of QuB.                                            */

/* QuB is free software; you can redistribute it and/or modify          */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation, either version 3 of the License, or    */
/* (at your option) any later version.                                  */

/* QuB is distributed in the hope that it will be useful,               */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */

/* You should have received a copy of the GNU General Public License,   */
/* named LICENSE.txt, in the QuB program directory.  If not, see        */
/* <http://www.gnu.org/licenses/>.                                      */

#ifdef _WIN32
  #include <windows.h>
#else
  #include <stdlib.h>
  #define BOOL int
  #define TRUE 1
  #define FALSE 0
#endif

#include <iostream>

#include "maxill_misc.h"
#include "ublas_plus.h"
#include "ublas_matrixutil.h"

extern "C" MAXILL_API void __stdcall MeanFirstPassage(double** A, double* pEq, double dt, int n, double** FL)
{
	// from http://www.icsu-scope.org/downloadpubs/scope34/contents.html
	//              Z = inv(I - A + col(1)*row(pEq))
	//       FL[i][j] = (Z[j][j] - Z[i][j]) / pEq[j]

	fq::matrix<double> M(n, n);
	for (int i=0; i<n; ++i)
		for (int j=0; j<n; ++j)
			M[i][j] = ((i==j)?1.0:0.0) - A[i][j] + pEq[j];
	
	fq::matrix<double> Z(n, n);
	sv_invm(M, n, Z);
	//gaussj0(M, n, NULL, 0, std::cerr);
	//for (int i=0; i<n; ++i)
	//	for (int j=0; j<n; ++j)
	//		Z[i][j] = M[i][j];

	for (int i=0; i<n; ++i)
		for (int j=0; j<n; ++j)
			FL[i][j] = dt * (Z[j][j] - Z[i][j]) / pEq[j];
}
